@@ -1,5 +1,5 @@
 /* Library function for scanning an archive file.
-Copyright (C) 1987-2020 Free Software Foundation, Inc.
+Copyright (C) 1987-2022 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify it under the
@@ -378,6 +378,31 @@ struct ar_hdr
 
 #include "output.h"
 
+
+static unsigned long int
+parse_int (const char *ptr, const size_t len, const int base,
+           const char *type, const char *archive, const char *name)
+{
+  const char *const ep = ptr + len;
+  const char max = '0' + base - 1;
+  long int val = 0;
+
+  /* In all the versions I know of the spaces come last, but be safe.  */
+  while (ptr < ep && *ptr == ' ')
+    ++ptr;
+
+  while (ptr < ep && *ptr != ' ')
+    {
+      if (*ptr < '0' || *ptr > max)
+        OSSS (fatal, NILF, _("Invalid %s for archive %s member %s"),
+                           type, archive, name);
+      val = (val * base) + (*ptr - '0');
+      ++ptr;
+    }
+
+  return val;
+}
+
 /* Takes three arguments ARCHIVE, FUNCTION and ARG.
 
    Open the archive named ARCHIVE, find its members one by one,
@@ -416,7 +441,7 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
 # endif
 #endif
   char *namemap = 0;
-  int namemap_size = 0;
+  unsigned int namemap_size = 0;
   int desc = open (archive, O_RDONLY, 0);
   if (desc < 0)
     return -1;
@@ -516,7 +541,7 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
 
     while (1)
       {
-        int nread;
+        ssize_t nread;
         struct ar_hdr member_header;
 #ifdef AIAMAGBIG
         struct ar_hdr_big member_header_big;
@@ -540,6 +565,8 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
         long int fnval;
         off_t o;
 
+        memset(&member_header, '\0', sizeof (member_header));
+
         EINTRLOOP (o, lseek (desc, member_offset, 0));
         if (o < 0)
           goto invalid;
@@ -669,10 +696,11 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
               && (name[0] == ' ' || name[0] == '/')
               && namemap != 0)
             {
-              int name_off = atoi (name + 1);
-              int name_len;
+              const char* err;
+              unsigned int name_off = make_toui (name + 1, &err);
+              size_t name_len;
 
-              if (name_off < 0 || name_off >= namemap_size)
+              if (err|| name_off >= namemap_size)
                 goto invalid;
 
               name = namemap + name_off;
@@ -685,9 +713,10 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
                    && name[1] == '1'
                    && name[2] == '/')
             {
-              int name_len = atoi (name + 3);
+              const char* err;
+              unsigned int name_len = make_toui (name + 3, &err);
 
-              if (name_len < 1)
+              if (err || name_len == 0 || name_len > PATH_MAX)
                 goto invalid;
 
               name = alloca (name_len + 1);
@@ -703,8 +732,8 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
         }
 
 #ifndef M_XENIX
-        sscanf (TOCHAR (member_header.ar_mode), "%8o", &eltmode);
-        eltsize = atol (TOCHAR (member_header.ar_size));
+        eltmode = parse_int (TOCHAR (member_header.ar_mode), sizeof (member_header.ar_mode), 8, "mode", archive, name);
+        eltsize = parse_int (TOCHAR (member_header.ar_size), sizeof (member_header.ar_size), 10, "size", archive, name);
 #else   /* Xenix.  */
         eltmode = (unsigned short int) member_header.ar_mode;
         eltsize = member_header.ar_size;
@@ -714,9 +743,9 @@ ar_scan (const char *archive, ar_member_func_t function, const void *arg)
           (*function) (desc, name, ! long_name, member_offset,
                        member_offset + AR_HDR_SIZE, eltsize,
 #ifndef M_XENIX
-                       atol (TOCHAR (member_header.ar_date)),
-                       atoi (TOCHAR (member_header.ar_uid)),
-                       atoi (TOCHAR (member_header.ar_gid)),
+                       parse_int (TOCHAR (member_header.ar_date), sizeof (member_header.ar_date), 10, "date", archive, name),
+                       parse_int (TOCHAR (member_header.ar_uid), sizeof (member_header.ar_uid), 10, "uid", archive, name),
+                       parse_int (TOCHAR (member_header.ar_gid), sizeof (member_header.ar_gid), 10, "gid", archive, name),
 #else   /* Xenix.  */
                        member_header.ar_date,
                        member_header.ar_uid,
