@@ -1,5 +1,5 @@
 /* Miscellaneous generic support functions for GNU Make.
-Copyright (C) 1988-2019 Free Software Foundation, Inc.
+Copyright (C) 1988-2022 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify it under the
@@ -17,6 +17,7 @@ this program.  If not, see <http://www.gnu.org/licenses/>.  */
 #include "makeint.h"
 #include "filedef.h"
 #include "dep.h"
+#include "os.h"
 #include "debug.h"
 
 /* GNU make no longer supports pre-ANSI89 environments.  */
@@ -24,6 +25,7 @@ this program.  If not, see <http://www.gnu.org/licenses/>.  */
 #include <stdarg.h>
 
 #ifdef WINDOWS32
+# include <windows.h>
 # include <io.h>
 #endif
 
@@ -33,6 +35,25 @@ this program.  If not, see <http://www.gnu.org/licenses/>.  */
 # include <sys/file.h>
 #endif
 
+unsigned int
+make_toui (const char *str, const char **error)
+{
+  char *end;
+  unsigned long val = strtoul (str, &end, 10);
+
+  if (error)
+    {
+      if (str[0] == '\0')
+        *error = "Missing value";
+      else if (*end != '\0')
+        *error = "Invalid value";
+      else
+        *error = NULL;
+    }
+
+  return val;
+}
+
 /* Compare strings *S1 and *S2.
    Return negative if the first is less, positive if it is greater,
    zero if they are equal.  */
@@ -172,6 +193,16 @@ concat (unsigned int num, ...)
   return result;
 }
 
+
+#ifndef HAVE_UNISTD_H
+pid_t getpid ();
+#endif
+
+pid_t make_pid ()
+{
+  return getpid ();
+}
+
 /* Like malloc but get fatal error if memory is exhausted.  */
 /* Don't bother if we're using dmalloc; it provides these for us.  */
 
@@ -260,6 +291,30 @@ xstrndup (const char *str, size_t length)
 
   return result;
 }
+
+#ifndef HAVE_MEMRCHR
+void *
+memrchr(const void* str, int ch, size_t len)
+{
+  const char* sp = str;
+  const char* cp = sp;
+
+  if (len == 0)
+    return NULL;
+
+  cp += len - 1;
+
+  while (cp[0] != ch)
+    {
+      if (cp == sp)
+        return NULL;
+      --cp;
+    }
+
+  return (void*)cp;
+}
+#endif
+
 
 
 /* Limited INDEX:
@@ -283,7 +338,8 @@ lindex (const char *s, const char *limit, int c)
 char *
 end_of_token (const char *s)
 {
-  END_OF_TOKEN (s);
+  while (! END_OF_TOKEN (*s))
+    ++s;
   return (char *)s;
 }
 
@@ -407,7 +463,8 @@ free_ns_chain (struct nameseq *ns)
 
 #ifdef MAKE_MAINTAINER_MODE
 
-void spin(const char* type)
+void
+spin (const char* type)
 {
   char filenm[256];
   struct stat dummy;
@@ -418,7 +475,11 @@ void spin(const char* type)
     {
       fprintf (stderr, "SPIN on %s\n", filenm);
       do
+#ifdef WINDOWS32
+        Sleep (1000);
+#else
         sleep (1);
+#endif
       while (stat (filenm, &dummy) == 0);
     }
 }
@@ -445,54 +506,155 @@ umask (mode_t mask)
 }
 #endif
 
-FILE *
-get_tmpfile (char **name, const char *template)
-{
-  FILE *file;
-#ifdef HAVE_FDOPEN
-  int fd;
+#define MAKE_TMPDIR         "MAKE_TMPDIR"
+#ifdef VMS
+# define DEFAULT_TMPFILE    "sys$scratch:gnv$make_cmdXXXXXX.com"
+# define DEFAULT_TMPDIR     "/sys$scratch/"
+#else
+# define DEFAULT_TMPFILE     "GmXXXXXX"
+# ifdef P_tmpdir
+#  define DEFAULT_TMPDIR    P_tmpdir
+# else
+#  define DEFAULT_TMPDIR    "/tmp"
+# endif
 #endif
 
+static const char *
+get_tmpdir ()
+{
+  static const char *tmpdir = NULL;
+
+  if (!tmpdir)
+    {
+      if (((tmpdir = getenv (MAKE_TMPDIR)) == NULL || *tmpdir == '\0')
+#if defined (__MSDOS__) || defined (WINDOWS32) || defined (__EMX__)
+          && ((tmpdir = getenv ("TMP")) == NULL || *tmpdir == '\0')
+          && ((tmpdir = getenv ("TEMP")) == NULL || *tmpdir == '\0')
+#endif
+          && ((tmpdir = getenv ("TMPDIR")) == NULL || *tmpdir == '\0'))
+        tmpdir = DEFAULT_TMPDIR;
+    }
+
+  return tmpdir;
+}
+
+static char *
+get_tmptemplate ()
+{
+  const char *tmpdir = get_tmpdir ();
+  char *template;
+  size_t len;
+
+  len = strlen (tmpdir);
+  template = xmalloc (len + CSTRLEN (DEFAULT_TMPFILE) + 2);
+  strcpy (template, tmpdir);
+
+#ifdef HAVE_DOS_PATHS
+  if (template[len - 1] != '/' && template[len - 1] != '\\')
+    strcat (template, "/");
+#else
+# ifndef VMS
+  if (template[len - 1] != '/')
+    strcat (template, "/");
+# endif /* !VMS */
+#endif /* !HAVE_DOS_PATHS */
+
+  strcat (template, DEFAULT_TMPFILE);
+
+  return template;
+}
+
+char *
+get_tmppath ()
+{
+  char *path;
+
+#ifdef HAVE_MKTEMP
+  path = get_tmptemplate();
+  if (*mktemp (path) == '\0')
+    pfatal_with_name ("mktemp");
+#else
+  path = xmalloc (L_tmpnam + 1);
+  if (tmpnam (path) == NULL)
+    pfatal_with_name ("tmpnam");
+#endif
+
+  return path;
+}
+
+/* Generate a temporary file and return an fd for it.  If name is NULL then
+   the temp file is anonymous and will be deleted when the process exits.  */
+int
+get_tmpfd (char **name)
+{
+  int fd = -1;
+  char *tmpnm;
+  mode_t mask;
+
+  /* If there's an os-specific way to get an anoymous temp file use it.  */
+  if (!name)
+    {
+      fd = os_anontmp ();
+      if (fd >= 0)
+        return fd;
+    }
+
   /* Preserve the current umask, and set a restrictive one for temp files.  */
-  mode_t mask = umask (0077);
+  mask = umask (0077);
 
-#if defined(HAVE_MKSTEMP) || defined(HAVE_MKTEMP)
-# define TEMPLATE_LEN   strlen (template)
-#else
-# define TEMPLATE_LEN   L_tmpnam
-#endif
-  *name = xmalloc (TEMPLATE_LEN + 1);
-  strcpy (*name, template);
+#if defined(HAVE_MKSTEMP)
+  tmpnm = get_tmptemplate ();
 
-#if defined(HAVE_MKSTEMP) && defined(HAVE_FDOPEN)
   /* It's safest to use mkstemp(), if we can.  */
-  EINTRLOOP (fd, mkstemp (*name));
-  if (fd == -1)
-    file = NULL;
-  else
-    file = fdopen (fd, "w");
+  EINTRLOOP (fd, mkstemp (tmpnm));
 #else
-# ifdef HAVE_MKTEMP
-  (void) mktemp (*name);
-# else
-  (void) tmpnam (*name);
-# endif
+  tmpnm = get_tmppath ();
 
-# ifdef HAVE_FDOPEN
-  /* Can't use mkstemp(), but guard against a race condition.  */
-  EINTRLOOP (fd, open (*name, O_CREAT|O_EXCL|O_WRONLY, 0600));
-  if (fd == -1)
-    return 0;
-  file = fdopen (fd, "w");
-# else
-  /* Not secure, but what can we do?  */
-  file = fopen (*name, "w");
-# endif
+  /* Can't use mkstemp(), but try to guard against a race condition.  */
+  EINTRLOOP (fd, open (tmpnm, O_CREAT|O_EXCL|O_RDWR, 0600));
 #endif
 
   umask (mask);
 
+  if (name)
+    *name = tmpnm;
+  else
+    {
+      unlink (tmpnm);
+      free (tmpnm);
+    }
+
+  return fd;
+}
+
+FILE *
+get_tmpfile (char **name)
+{
+#if defined(HAVE_FDOPEN)
+  int fd = get_tmpfd (name);
+
+  return fd < 0 ? NULL : fdopen (fd, "w");
+#else
+  /* Preserve the current umask, and set a restrictive one for temp files.  */
+  mode_t mask = umask (0077);
+
+  char *tmpnm = get_tmppath ();
+
+  /* Not secure, but...?  If name is NULL we could use tmpfile()...  */
+  FILE *file = fopen (tmpnm, "w");
+
+  umask (mask);
+
+  if (name)
+    *name = tmpnm;
+  else
+    {
+      unlink (tmpnm);
+      free (tmpnm);
+    }
+
   return file;
+#endif
 }
 
 
@@ -505,8 +667,8 @@ strcasecmp (const char *s1, const char *s2)
 {
   while (1)
     {
-      int c1 = (int) *(s1++);
-      int c2 = (int) *(s2++);
+      int c1 = (unsigned char) *(s1++);
+      int c2 = (unsigned char) *(s2++);
 
       if (isalpha (c1))
         c1 = tolower (c1);
@@ -530,8 +692,8 @@ strncasecmp (const char *s1, const char *s2, int n)
 {
   while (n-- > 0)
     {
-      int c1 = (int) *(s1++);
-      int c2 = (int) *(s2++);
+      int c1 = (unsigned char) *(s1++);
+      int c2 = (unsigned char) *(s2++);
 
       if (isalpha (c1))
         c1 = tolower (c1);
@@ -548,254 +710,6 @@ strncasecmp (const char *s1, const char *s2, int n)
 }
 #endif
 
-#ifdef  GETLOADAVG_PRIVILEGED
-
-#ifdef POSIX
-
-/* Hopefully if a system says it's POSIX.1 and has the setuid and setgid
-   functions, they work as POSIX.1 says.  Some systems (Alpha OSF/1 1.2,
-   for example) which claim to be POSIX.1 also have the BSD setreuid and
-   setregid functions, but they don't work as in BSD and only the POSIX.1
-   way works.  */
-
-#undef HAVE_SETREUID
-#undef HAVE_SETREGID
-
-#else   /* Not POSIX.  */
-
-/* Some POSIX.1 systems have the seteuid and setegid functions.  In a
-   POSIX-like system, they are the best thing to use.  However, some
-   non-POSIX systems have them too but they do not work in the POSIX style
-   and we must use setreuid and setregid instead.  */
-
-#undef HAVE_SETEUID
-#undef HAVE_SETEGID
-
-#endif  /* POSIX.  */
-
-#ifndef HAVE_UNISTD_H
-extern int getuid (), getgid (), geteuid (), getegid ();
-extern int setuid (), setgid ();
-#ifdef HAVE_SETEUID
-extern int seteuid ();
-#else
-#ifdef  HAVE_SETREUID
-extern int setreuid ();
-#endif  /* Have setreuid.  */
-#endif  /* Have seteuid.  */
-#ifdef HAVE_SETEGID
-extern int setegid ();
-#else
-#ifdef  HAVE_SETREGID
-extern int setregid ();
-#endif  /* Have setregid.  */
-#endif  /* Have setegid.  */
-#endif  /* No <unistd.h>.  */
-
-/* Keep track of the user and group IDs for user- and make- access.  */
-static int user_uid = -1, user_gid = -1, make_uid = -1, make_gid = -1;
-#define access_inited   (user_uid != -1)
-static enum { make, user } current_access;
-
-
-/* Under -d, write a message describing the current IDs.  */
-
-static void
-log_access (const char *flavor)
-{
-  if (! ISDB (DB_JOBS))
-    return;
-
-  /* All the other debugging messages go to stdout,
-     but we write this one to stderr because it might be
-     run in a child fork whose stdout is piped.  */
-
-  fprintf (stderr, _("%s: user %lu (real %lu), group %lu (real %lu)\n"),
-           flavor, (unsigned long) geteuid (), (unsigned long) getuid (),
-           (unsigned long) getegid (), (unsigned long) getgid ());
-  fflush (stderr);
-}
-
-
-static void
-init_access (void)
-{
-#ifndef VMS
-  user_uid = getuid ();
-  user_gid = getgid ();
-
-  make_uid = geteuid ();
-  make_gid = getegid ();
-
-  /* Do these ever fail?  */
-  if (user_uid == -1 || user_gid == -1 || make_uid == -1 || make_gid == -1)
-    pfatal_with_name ("get{e}[gu]id");
-
-  log_access (_("Initialized access"));
-
-  current_access = make;
-#endif
-}
-
-#endif  /* GETLOADAVG_PRIVILEGED */
-
-/* Give the process appropriate permissions for access to
-   user data (i.e., to stat files, or to spawn a child process).  */
-void
-user_access (void)
-{
-#ifdef  GETLOADAVG_PRIVILEGED
-
-  if (!access_inited)
-    init_access ();
-
-  if (current_access == user)
-    return;
-
-  /* We are in "make access" mode.  This means that the effective user and
-     group IDs are those of make (if it was installed setuid or setgid).
-     We now want to set the effective user and group IDs to the real IDs,
-     which are the IDs of the process that exec'd make.  */
-
-#ifdef  HAVE_SETEUID
-
-  /* Modern systems have the seteuid/setegid calls which set only the
-     effective IDs, which is ideal.  */
-
-  if (seteuid (user_uid) < 0)
-    pfatal_with_name ("user_access: seteuid");
-
-#else   /* Not HAVE_SETEUID.  */
-
-#ifndef HAVE_SETREUID
-
-  /* System V has only the setuid/setgid calls to set user/group IDs.
-     There is an effective ID, which can be set by setuid/setgid.
-     It can be set (unless you are root) only to either what it already is
-     (returned by geteuid/getegid, now in make_uid/make_gid),
-     the real ID (return by getuid/getgid, now in user_uid/user_gid),
-     or the saved set ID (what the effective ID was before this set-ID
-     executable (make) was exec'd).  */
-
-  if (setuid (user_uid) < 0)
-    pfatal_with_name ("user_access: setuid");
-
-#else   /* HAVE_SETREUID.  */
-
-  /* In 4BSD, the setreuid/setregid calls set both the real and effective IDs.
-     They may be set to themselves or each other.  So you have two alternatives
-     at any one time.  If you use setuid/setgid, the effective will be set to
-     the real, leaving only one alternative.  Using setreuid/setregid, however,
-     you can toggle between your two alternatives by swapping the values in a
-     single setreuid or setregid call.  */
-
-  if (setreuid (make_uid, user_uid) < 0)
-    pfatal_with_name ("user_access: setreuid");
-
-#endif  /* Not HAVE_SETREUID.  */
-#endif  /* HAVE_SETEUID.  */
-
-#ifdef  HAVE_SETEGID
-  if (setegid (user_gid) < 0)
-    pfatal_with_name ("user_access: setegid");
-#else
-#ifndef HAVE_SETREGID
-  if (setgid (user_gid) < 0)
-    pfatal_with_name ("user_access: setgid");
-#else
-  if (setregid (make_gid, user_gid) < 0)
-    pfatal_with_name ("user_access: setregid");
-#endif
-#endif
-
-  current_access = user;
-
-  log_access (_("User access"));
-
-#endif  /* GETLOADAVG_PRIVILEGED */
-}
-
-/* Give the process appropriate permissions for access to
-   make data (i.e., the load average).  */
-void
-make_access (void)
-{
-#ifdef  GETLOADAVG_PRIVILEGED
-
-  if (!access_inited)
-    init_access ();
-
-  if (current_access == make)
-    return;
-
-  /* See comments in user_access, above.  */
-
-#ifdef  HAVE_SETEUID
-  if (seteuid (make_uid) < 0)
-    pfatal_with_name ("make_access: seteuid");
-#else
-#ifndef HAVE_SETREUID
-  if (setuid (make_uid) < 0)
-    pfatal_with_name ("make_access: setuid");
-#else
-  if (setreuid (user_uid, make_uid) < 0)
-    pfatal_with_name ("make_access: setreuid");
-#endif
-#endif
-
-#ifdef  HAVE_SETEGID
-  if (setegid (make_gid) < 0)
-    pfatal_with_name ("make_access: setegid");
-#else
-#ifndef HAVE_SETREGID
-  if (setgid (make_gid) < 0)
-    pfatal_with_name ("make_access: setgid");
-#else
-  if (setregid (user_gid, make_gid) < 0)
-    pfatal_with_name ("make_access: setregid");
-#endif
-#endif
-
-  current_access = make;
-
-  log_access (_("Make access"));
-
-#endif  /* GETLOADAVG_PRIVILEGED */
-}
-
-/* Give the process appropriate permissions for a child process.
-   This is like user_access, but you can't get back to make_access.  */
-void
-child_access (void)
-{
-#ifdef  GETLOADAVG_PRIVILEGED
-
-  if (!access_inited)
-    abort ();
-
-  /* Set both the real and effective UID and GID to the user's.
-     They cannot be changed back to make's.  */
-
-#ifndef HAVE_SETREUID
-  if (setuid (user_uid) < 0)
-    pfatal_with_name ("child_access: setuid");
-#else
-  if (setreuid (user_uid, user_uid) < 0)
-    pfatal_with_name ("child_access: setreuid");
-#endif
-
-#ifndef HAVE_SETREGID
-  if (setgid (user_gid) < 0)
-    pfatal_with_name ("child_access: setgid");
-#else
-  if (setregid (user_gid, user_gid) < 0)
-    pfatal_with_name ("child_access: setregid");
-#endif
-
-  log_access (_("Child access"));
-
-#endif  /* GETLOADAVG_PRIVILEGED */
-}
 
 #ifdef NEED_GET_PATH_MAX
 unsigned int
@@ -815,3 +729,153 @@ get_path_max (void)
   return value;
 }
 #endif
+
+#if !HAVE_MEMPCPY
+void *
+mempcpy (void *dest, const void *src, size_t n)
+{
+  return (char *) memcpy (dest, src, n) + n;
+}
+#endif
+
+#if !HAVE_STPCPY
+char *
+stpcpy (char *dest, const char *src)
+{
+  char *d = dest;
+  const char *s = src;
+
+  do
+    *d++ = *s;
+  while (*s++ != '\0');
+
+  return d - 1;
+}
+#endif
+
+#if !HAVE_STRTOLL
+# undef UNSIGNED
+# undef USE_NUMBER_GROUPING
+# undef USE_WIDE_CHAR
+# define QUAD 1
+# include <strtol.c>
+#endif
+
+#if !HAVE_STRERROR
+char *
+strerror (int errnum)
+{
+  static char msg[256];
+
+#define SETMSG(_e, _m) case _e: strcpy(msg, _m); break
+
+  switch (errnum)
+    {
+#ifdef EPERM
+    SETMSG (EPERM  , "Operation not permitted");
+#endif
+#ifdef ENOENT
+    SETMSG (ENOENT , "No such file or directory");
+#endif
+#ifdef ESRCH
+    SETMSG (ESRCH  , "No such process");
+#endif
+#ifdef EINTR
+    SETMSG (EINTR  , "Interrupted system call");
+#endif
+#ifdef EIO
+    SETMSG (EIO    , "I/O error");
+#endif
+#ifdef ENXIO
+    SETMSG (ENXIO  , "No such device or address");
+#endif
+#ifdef E2BIG
+    SETMSG (E2BIG  , "Argument list too long");
+#endif
+#ifdef ENOEXEC
+    SETMSG (ENOEXEC, "Exec format error");
+#endif
+#ifdef EBADF
+    SETMSG (EBADF  , "Bad file number");
+#endif
+#ifdef ECHILD
+    SETMSG (ECHILD , "No child processes");
+#endif
+#ifdef EAGAIN
+    SETMSG (EAGAIN , "Try again");
+#endif
+#ifdef ENOMEM
+    SETMSG (ENOMEM , "Out of memory");
+#endif
+#ifdef EACCES
+    SETMSG (EACCES , "Permission denied");
+#endif
+#ifdef EFAULT
+    SETMSG (EFAULT , "Bad address");
+#endif
+#ifdef ENOTBLK
+    SETMSG (ENOTBLK, "Block device required");
+#endif
+#ifdef EBUSY
+    SETMSG (EBUSY  , "Device or resource busy");
+#endif
+#ifdef EEXIST
+    SETMSG (EEXIST , "File exists");
+#endif
+#ifdef EXDEV
+    SETMSG (EXDEV  , "Cross-device link");
+#endif
+#ifdef ENODEV
+    SETMSG (ENODEV , "No such device");
+#endif
+#ifdef ENOTDIR
+    SETMSG (ENOTDIR, "Not a directory");
+#endif
+#ifdef EISDIR
+    SETMSG (EISDIR , "Is a directory");
+#endif
+#ifdef EINVAL
+    SETMSG (EINVAL , "Invalid argument");
+#endif
+#ifdef ENFILE
+    SETMSG (ENFILE , "File table overflow");
+#endif
+#ifdef EMFILE
+    SETMSG (EMFILE , "Too many open files");
+#endif
+#ifdef ENOTTY
+    SETMSG (ENOTTY , "Not a typewriter");
+#endif
+#ifdef ETXTBSY
+    SETMSG (ETXTBSY, "Text file busy");
+#endif
+#ifdef EFBIG
+    SETMSG (EFBIG  , "File too large");
+#endif
+#ifdef ENOSPC
+    SETMSG (ENOSPC , "No space left on device");
+#endif
+#ifdef ESPIPE
+    SETMSG (ESPIPE , "Illegal seek");
+#endif
+#ifdef EROFS
+    SETMSG (EROFS  , "Read-only file system");
+#endif
+#ifdef EMLINK
+    SETMSG (EMLINK , "Too many links");
+#endif
+#ifdef EPIPE
+    SETMSG (EPIPE  , "Broken pipe");
+#endif
+#ifdef EDOM
+    SETMSG (EDOM   , "Math argument out of domain of func");
+#endif
+#ifdef ERANGE
+    SETMSG (ERANGE , "Math result not representable");
+#endif
+    default: sprintf (msg, "Unknown error %d", errnum); break;
+    }
+
+  return msg;
+}
+#endif
